@@ -1,15 +1,18 @@
 import shutil
+import pandas as pd
+
+import torchero
+
 from .common import *
 
-
 class HistoryCallbackTests(unittest.TestCase):
-    def test_history_callback_register_every_training_stat(self):
+    def setUp(self):
         self.model = DummyModel()
         train_dataset = torch.arange(10).view(-1, 1)
         valid_dataset = torch.arange(10).view(-1, 1)
 
-        train_dl = DataLoader(train_dataset, shuffle=False, batch_size=1)
-        valid_dl = DataLoader(valid_dataset, shuffle=False, batch_size=1)
+        self.train_dl = DataLoader(train_dataset, shuffle=False, batch_size=1)
+        self.valid_dl = DataLoader(valid_dataset, shuffle=False, batch_size=1)
 
         def update_batch(trainer, x):
             trainer.train_meters['t_c'].measure(x.data[0][0])
@@ -17,23 +20,52 @@ class HistoryCallbackTests(unittest.TestCase):
         def validate_batch(validator, x):
             validator.meters['v_c'].measure(x.data[0][0])
 
-        trainer = TestTrainer(model=self.model,
-                              logging_frecuency=5,
-                              train_meters={'t_c' : Averager()},
-                              val_meters={'v_c' : Averager()},
-                              update_batch_fn=update_batch,
-                              valid_batch_fn=validate_batch)
-        self.assertEqual(trainer.metrics, {})
+        self.trainer = TestTrainer(model=self.model,
+                                   logging_frecuency=5,
+                                   train_meters={'t_c' : Averager()},
+                                   val_meters={'v_c' : Averager()},
+                                   update_batch_fn=update_batch,
+                                   valid_batch_fn=validate_batch)
 
-        self.assertEqual(set(trainer.meters_names()), set(['t_c', 'v_c']))
+    def test_repr(self):
+        self.assertEqual(repr(History()), "History()")
 
-        trainer.train(train_dl, valid_dataloader=valid_dl, epochs=1)
+    def test_history_callback_register_every_training_stat(self):
+        self.assertEqual(self.trainer.metrics, {})
+
+        self.assertEqual(set(self.trainer.meters_names()), set(['t_c', 'v_c']))
+
+        self.trainer.train(self.train_dl, valid_dataloader=self.valid_dl, epochs=1)
 
         expected_registry = [{'epoch': 0, 'step': 5, 't_c': 2.0, 'v_c': 4.5},
-                             {'epoch': 0, 'step': 10, 't_c': 7.0, 'v_c': 4.5}]
+                             {'epoch': 0, 'step': 10, 't_c': 4.5, 'v_c': 4.5}]
 
-        self.assertEqual(list(trainer.history), expected_registry)
-        self.assertEqual(trainer.metrics, {'t_c': 7.0, 'v_c': 4.5})
+        self.assertEqual(list(self.trainer.history), expected_registry)
+        self.assertEqual(self.trainer.metrics, {'t_c': 4.5, 'v_c': 4.5})
+
+    def test_export_to_dataframe(self):
+        self.trainer.train(self.train_dl, valid_dataloader=self.valid_dl, epochs=2)
+
+        try:
+            self.trainer.history.to_dataframe(level='xxx')
+            self.fail()
+        except Exception as e:
+            self.assertIsInstance(e, ValueError)
+            self.assertEqual(str(e), self.trainer.history.UNRECOGNIZED_LEVEL_MESSAGE.format(level=repr('xxx')))
+
+        df_history = self.trainer.history.to_dataframe(level='step')
+        self.assertIsInstance(df_history, pd.DataFrame)
+        df_expected = pd.DataFrame({'epoch': [0, 0, 1, 1],
+                                    'step':  [5, 10, 15, 20],
+                                    't_c':   [2.0, 4.5, 2.0, 4.5],
+                                    'v_c':   [4.5, 4.5, 4.5, 4.5]})
+        pd.testing.assert_frame_equal(df_history, df_expected)
+
+        df_history = self.trainer.history.to_dataframe(level='epoch')
+        df_expected = pd.DataFrame({'epoch': [0, 1],
+                                    't_c':   [4.5, 4.5],
+                                    'v_c':   [4.5, 4.5]})
+        pd.testing.assert_frame_equal(df_history, df_expected)
 
 
 class CSVExporterTests(unittest.TestCase):
@@ -122,13 +154,13 @@ class CSVExporterTests(unittest.TestCase):
             self.assertEqual(len(lines), 3)
             self.assertEqual(lines[0], 'epoch,step,c\n')
             self.assertEqual(lines[1], '0,5,2.0\n')
-            self.assertEqual(lines[2], '0,10,7.0')
+            self.assertEqual(lines[2], '0,10,4.5')
 
         with open(self.stats_filename_epoch, 'r') as f:
             lines = f.readlines()
             self.assertEqual(len(lines), 2)
             self.assertEqual(lines[0], 'epoch,c\n')
-            self.assertEqual(lines[1], '1,7.0')
+            self.assertEqual(lines[1], '1,4.5')
 
 
 
@@ -153,16 +185,16 @@ class CSVExporterTests(unittest.TestCase):
             self.assertEqual(len(lines), 5)
             self.assertEqual(lines[0], 'epoch,step,c\n')
             self.assertEqual(lines[1], '0,5,2.0\n')
-            self.assertEqual(lines[2], '0,10,7.0\n')
+            self.assertEqual(lines[2], '0,10,4.5\n')
             self.assertEqual(lines[3], '1,15,2.0\n')
-            self.assertEqual(lines[4], '1,20,7.0')
+            self.assertEqual(lines[4], '1,20,4.5')
 
         with open(self.stats_filename_epoch, 'r') as f:
             lines = f.readlines()
             self.assertEqual(len(lines), 3)
             self.assertEqual(lines[0], 'epoch,c\n')
-            self.assertEqual(lines[1], '1,7.0\n')
-            self.assertEqual(lines[2], '2,7.0')
+            self.assertEqual(lines[1], '1,4.5\n')
+            self.assertEqual(lines[2], '2,4.5')
 
     def test_csv_exporter_exports_only_selected_columns(self):
         self.load_arange_dataset()
@@ -200,7 +232,19 @@ class CSVExporterTests(unittest.TestCase):
             self.assertEqual(len(lines), 3)
             self.assertEqual(lines[0], 'epoch,v,c\n')
             self.assertEqual(lines[1], '0,,2.0\n')
-            self.assertEqual(lines[2], '0,,7.0')
+            self.assertEqual(lines[2], '0,,4.5')
+
+    def test_repr(self):
+        self.assertEqual(repr(CSVLogger(output='stats.csv',
+                                        append=False,
+                                        level='epoch')),
+                         "CSVLogger(output='stats.csv', append=False, level='epoch')")
+        self.assertEqual(repr(CSVLogger(output='stats.csv',
+                                        append=True,
+                                        level='step',
+                                        columns=['train_acc'],
+                                        hparams_columns=['lr'])),
+                         "CSVLogger(output='stats.csv', append=True, level='step', columns=['train_acc'], hparams_columns=['lr'])")
 
     def tearDown(self):
         shutil.rmtree(self.base_tree)
@@ -232,6 +276,12 @@ class ProgbarTests(unittest.TestCase):
         self.assertEqual(self.callback.epoch_tqdm.unit, 'epoch')
         self.assertEqual(self.callback.epoch_tqdm.total, 2)
 
+    def test_repr(self):
+        self.assertEqual(repr(ProgbarLogger(ascii=True, notebook=False)),
+                         'ProgbarLogger(ascii=True, notebook=False)')
+        self.assertEqual(repr(ProgbarLogger(ascii=False, notebook=True, monitors=['train_acc'])),
+                         'ProgbarLogger(ascii=False, notebook=True, monitors=[\'train_acc\'])')
+
 
 class CheckpointTests(unittest.TestCase):
     def setUp(self):
@@ -521,8 +571,9 @@ class CheckpointTests(unittest.TestCase):
                               update_batch_fn=self.meter_from_list(measures, 't'))
         try:
             trainer.train(self.train_dl, epochs=0)
+            self.fail()
         except Exception as e:
-            self.assertEqual(str(e), checkpoint.INVALID_MODE_INFERENCE_MESSAGE.format(meter='t'))
+            self.assertEqual(str(e), torchero.utils.defaults.INVALID_MODE_INFERENCE_MESSAGE.format(meter='Averager'))
 
 class EarlyStoppingTests(unittest.TestCase):
     def setUp(self):
@@ -541,6 +592,10 @@ class EarlyStoppingTests(unittest.TestCase):
                                patience=patience,
                                min_delta=min_delta)
 
+    def test_repr(self):
+        self.assertEqual(repr(EarlyStopping(monitor='train_acc', min_delta=0.1, patience=1, mode='min')),
+                         "EarlyStopping(monitor='train_acc', min_delta=0.1, patience=1, mode='min')")
+
     @staticmethod
     def meter_from_list(measure_list, meter_name):
         def update_batch_fn(trainer, _):
@@ -691,5 +746,6 @@ class EarlyStoppingTests(unittest.TestCase):
                               update_batch_fn=self.meter_from_list(measures, 'v'))
         try:
             trainer.train(self.train_dl, epochs=0)
+            self.fail()
         except Exception as e:
-            self.assertEqual(str(e), callback.INVALID_MODE_INFERENCE_MESSAGE.format(meter='v'))
+            self.assertEqual(str(e), torchero.utils.defaults.INVALID_MODE_INFERENCE_MESSAGE.format(meter='Averager'))
