@@ -29,46 +29,6 @@ class TrainerTests(unittest.TestCase):
                                                 torch.zeros(end-start, 1).float())
         self.validation_dataloader = DataLoader(self.validation_dataset, batch_size=batch_size)
 
-    def test_train_val_loss_are_calculated_after_every_log_event(self):
-        trainer = SupervisedTrainer(model=self.model, optimizer=self.optimizer, criterion=self.criterion, logging_frecuency=1, validation_granularity=ValidationGranularity.AT_LOG)
-        train_meters = trainer.train_meters
-        val_meters = trainer.val_meters
-        meters = trainer.meters
-
-        self.assertListEqual(sorted(train_meters.keys()), ['loss'])
-        self.assertListEqual(sorted(meters.keys()), ['train_loss', 'val_loss'])
-
-        self.assertIsInstance(meters['train_loss'], LossMeter)
-        self.assertIsInstance(meters['val_loss'], LossMeter)
-        self.assertEqual(meters['train_loss'].criterion, self.criterion)
-        self.assertEqual(meters['val_loss'].criterion, self.criterion)
-        self.assertIs(train_meters['loss'], meters['train_loss'])
-        self.assertIs(val_meters['loss'], meters['val_loss'])
-
-        trainer.train(self.training_dataloader, valid_dataloader=self.validation_dataloader, epochs=1)
-
-        self.assertEqual(len(trainer.history), 2)
-
-        for i in range(2):
-            keys = ('epoch', 'step', 'train_loss', 'val_loss')
-            self.assertEqual(len(trainer.history[i]), len(keys))
-            for name in keys:
-                self.assertIn(name, trainer.history[i].keys())
-
-        self.assertEqual(trainer.history[0]['epoch'], 0)
-        self.assertEqual(trainer.history[0]['step'], 1)
-        # first train batch loss is calculated before weights update
-        self.assertAlmostEqual(trainer.history[0]['train_loss'], sum(abs(self.w*i) for i in range(1, 3))/2)
-        # Validation loss is calculated over the entire valid dataset after weights update
-        self.w -= (sign(self.w*1-0)*1 + sign(self.w*2-0)*2)/2.0
-        self.assertAlmostEqual(trainer.history[0]['val_loss'], sum(abs(self.w*i) for i in range(0,10))/10)
-
-        self.assertEqual(trainer.history[1]['epoch'], 0)
-        self.assertEqual(trainer.history[1]['step'], 2)
-        self.assertAlmostEqual(trainer.history[1]['train_loss'], sum(abs(self.w*i) for i in range(3, 5))/2)
-        self.w -= (sign(self.w*3-0)*3 + sign(self.w*4-0)*4)/2.0
-        self.assertAlmostEqual(trainer.history[1]['val_loss'], sum(abs(self.w*i) for i in range(0,10))/10)
-        self.assertAlmostEqual(self.model.weight.data[0][0], self.w)
 
     def test_trainer_with_acc_meter_argument_measure_train_and_valid_accuracy_with_same_metric(self):
         acc_meter = MSE()
@@ -93,13 +53,34 @@ class TrainerTests(unittest.TestCase):
             for name in keys:
                 self.assertIn(name, trainer.history[i].keys())
 
-        self.assertAlmostEqual(trainer.history[0]['train_acc'], sum((self.w*i)**2 for i in range(1, 3))/2)
+        first_loss = sum((self.w*i)**2 for i in range(1, 3))
+        self.assertAlmostEqual(trainer.history[0]['train_acc'], first_loss/2)
         self.w -= (sign(self.w*1-0)*1 + sign(self.w*2-0)*2)/2.0
         self.assertAlmostEqual(trainer.history[0]['val_acc'], sum((self.w*i)**2 for i in range(0,10))/10)
-        self.assertAlmostEqual(trainer.history[1]['train_acc'], sum((self.w*i)**2 for i in range(3, 5))/2)
+        second_loss = sum((self.w*i)**2 for i in range(3, 5))
+        self.assertAlmostEqual(trainer.history[1]['train_acc'], (first_loss + second_loss)/4)
         self.w -= (sign(self.w*3-0)*3 + sign(self.w*4-0)*4)/2.0
         self.assertAlmostEqual(trainer.history[1]['val_acc'], sum((self.w*i)**2 for i in range(0,10))/10)
 
+    def test_column_layout(self):
+        acc_meter = MSE()
+        trainer = SupervisedTrainer(model=self.model, optimizer=self.optimizer, acc_meters={'acc': acc_meter}, criterion=self.criterion, logging_frecuency=1, validation_granularity=ValidationGranularity.AT_LOG)
+        trainer.train(self.training_dataloader, valid_dataloader=self.validation_dataloader, epochs=2)
+        col_layout = trainer.history.get_column_layout()
+        expected_layout = { (0,0): { "metrics": "train_acc",
+                                     "title": "train_acc",
+                                     "ylabel": "acc" },
+                            (0,1): { "metrics": "val_acc",
+                                     "title": "val_acc",
+                                     "ylabel": "acc" },
+                            (1,0): { "metrics": "train_loss",
+                                     "title": "train_loss",
+                                     "ylabel": "loss" },
+                            (1,1): { "metrics": "val_loss",
+                                     "title": "val_loss",
+                                     "ylabel": "loss" }}
+        self.assertEqual(col_layout, expected_layout)
+
     def test_trainer_with_val_acc_meter_argument_cant_differ_from_train_acc_meter(self):
         acc_meter = MSE()
         val_acc_meter = RMSE()
@@ -137,6 +118,23 @@ class BinaryClassificationTrainerTest(unittest.TestCase):
         self.assertNotIn('train_precision_wl', trainer.metrics.keys())
         self.assertNotIn('train_recall_wl', trainer.metrics.keys())
         self.assertNotIn('train_f1_wl', trainer.metrics.keys())
+        default_layout = trainer.history.get_default_layout()
+        expected_layout = { (0,0): { "metrics": ["train_acc", "val_acc"],
+                                     "title": "train_acc/val_acc",
+                                     "ylabel": "acc"},
+                            (0,1): { "metrics": ["train_f1", "val_f1"],
+                                     "title": "train_f1/val_f1",
+                                     "ylabel": "f1" },
+                            (1,0): { "metrics": ["train_loss","val_loss"],
+                                     "title": "train_loss/val_loss",
+                                     "ylabel": "loss" },
+                            (1,1): { "metrics": ["train_precision","val_precision"],
+                                     "title": "train_precision/val_precision",
+                                     "ylabel": "precision"},
+                            (2,0): { "metrics": ["train_recall","val_recall"],
+                                     "title": "train_recall/val_recall",
+                                     "ylabel": "recall" } }
+        self.assertEqual(default_layout, expected_layout)
         metrics = trainer.evaluate(self.val_dl)
         self.assertIn('loss', metrics)
         self.assertIn('acc', metrics)
@@ -177,14 +175,16 @@ class UnsupervisedTrainerTests(unittest.TestCase):
         self.assertEqual(trainer.history[0]['epoch'], 0)
         self.assertEqual(trainer.history[0]['step'], 1)
         # first train batch loss is calculated before weights update
-        self.assertAlmostEqual(trainer.history[0]['train_loss'], sum(abs(self.w*i -  i) for i in range(1, 3))/2)
+        first_loss = sum(abs(self.w*i -  i) for i in range(1, 3))
+        self.assertAlmostEqual(trainer.history[0]['train_loss'], first_loss/2)
         # Validation loss is calculated over the entire valid dataset after weights update
         self.w -= (sign(self.w*1-1)*1 + sign(self.w*2-2)*2)/2.0
         self.assertAlmostEqual(trainer.history[0]['val_loss'], sum(abs(self.w*i - i) for i in range(0,10))/10)
 
+        second_loss = sum(abs(self.w*i -  i) for i in range(3, 5))
         self.assertEqual(trainer.history[1]['epoch'], 0)
         self.assertEqual(trainer.history[1]['step'], 2)
-        self.assertAlmostEqual(trainer.history[1]['train_loss'], sum(abs(self.w*i - i) for i in range(3, 5))/2)
+        self.assertAlmostEqual(trainer.history[1]['train_loss'], (first_loss + second_loss)/4)
         self.w -= (sign(self.w*3-3)*3 + sign(self.w*4-4)*4)/2.0
         self.assertAlmostEqual(trainer.history[1]['val_loss'], sum(abs(self.w*i - i) for i in range(0,10))/10)
         self.assertAlmostEqual(self.model.weight.data[0][0], self.w)
