@@ -33,3 +33,9 @@ class CallbackContainer(Callback):
 
     def on_train_end(self):
         self.signal('on_train_end')
+
+    def __repr__(self):
+        return repr(list(self.callbacks))
+
+    def __iter__(self):
+        return iter(self.callbacks)
