@@ -1,4 +1,4 @@
-from . import Dataset
+from torchero.utils.data.datasets import Dataset
 
 
 class UnsuperviseDataset(Dataset):
@@ -24,3 +24,13 @@ class UnsuperviseDataset(Dataset):
 
     def __len__(self):
         return len(self._dataset)
+
+    def __getattr__(self, attr):
+        return getattr(self._dataset, attr)
+
+    def __str__(self):
+        return str(self._dataset)
+
+    def __repr__(self):
+        return "{} of \n {}".format(self.__class__.__name__,
+                                    repr(self._dataset))
